/******************************************************************************

    B A S S   T A B   C R E A T O R
    ===============================

    Copyright (C) 1991/1992 by Mario D'Alessio
    Version 2.58  All Rights Reserved


    Your Distribution Rights
    ------------------------
    The Bass Tab Creator is protected by the United States copyright
    law.  It is copyright (c) 1991/1992 by Mario D'Alessio.  This
    product may be distributed freely to users as long as the product
    is not changed or altered and contains this documentation
    header.  However, it may be changed or altered in any way as long
    as the modified version is not distributed.  It may not be sold
    or packaged, either individually or as part of any other product,
    without the express written consent of the author listed above.

    The Bass Tab Creator is offered AS IS and absolutely NO
    guarantees are given or implied as to its suitability.  The
    author assumes no responsibility for any damage to your machine
    or loss of data incurred while using this program.


    *****************************************************************
    **                                                             **
    **  IMPORTANT PROGRAM NOTES:                                   **
    **  ------------------------                                   **
    **  - Below there is a #define MACHINE line which can be set   **
    **    for UNIX, MSDOS or ATARI_ST machines. Before compiling,  **
    **    set MACHINE to the proper value.                         **
    **                                                             **
    **  - There are other #define's that can be reconfigured.      **
    **    This is to allow a high degree of customization. Be      **
    **    sure to check these values.                              **
    **                                                             **
    **  - CUSTOM FONT 1 and 2 are meant to be used with another    **
    **    program called "MusicFont" (also written by Mario).      **
    **    This program redefines some of the characters on Epson   **
    **    printers to print actual notes instead letters for the   **
    **    duration values.                                         **
    **                                                             **
    **  - Since the characters that are detected and printed for   **
    **    the different features can be redefined, the information **
    **    in this header applies only to the original code.        **
    **                                                             **
    **  - This program has minimal error checking.                 **
    **                                                             **
    *****************************************************************


Usage:
======
    <program_name> [ -l line_length ] [-?] <input_files[.btc]>

    Output written to <input_file>.tab


Syntax:
=======
    FIRST LINE:           The first line of a .btc file is a title line.
                          This line is printed at the top of the resulting
                          .tab file. It is useful for song titles and artists.

                          The line can me only up to MAX_INPUT_LINE characters
                          in length.

    Comment Line:   #     Up to MAX_INPUT_LINE characters in length.
                          This line DOES NOT get printed.

                          Example:    # Vocals start here

    Note:
        <string> <fret> [<duration>]        --> Separated by ONE space

            string = String Number, 1 to 4

            fret   = Fret Number

            dur    = Note Duration   ** OPTIONAL **
                     w = Whole              W = Whole DOTTED
                     h = Half               H = Half DOTTED
                     q = Quarter            Q = Quarter DOTTED
                     e = Eighth             E = Eighth DOTTED
                     s = Sixteenth          S = Sixteenth DOTTED
                     t = Thirty-Second      T = Thirty-Second DOTTED
                     x = Sixty-Fourth       X = Sixty-Fourth DOTTED

        Examples:    1 12 e  --> String 1, Fret 12, Eighth note
                     3 4 H   --> String 3, Fret 4,  Dotted half note
                     2 3     --> String 3, Fret 4,  No duration printed

    Parenthesized Note:
        ( <string> <fret> [<duration>]      --> Separated by ONE space

                                   e
                     Example:   +-----+  is entered as:  ( 2 3 e
                                |-----|
                                |-(3)-|
                                +-----+

    Rest:
        r <dur>     dur = Rest Duration (see Note Duration above)
                    Separated by ONE space

                    Example:   r q --> Quarter rest

    New Measure:
        - [r,R] [<times to repeat>]
                    r = Begin OR End Repeat  ** OPTIONAL **
                    R = Begin AND End Repeat ** OPTIONAL **
                    times to repeat = Number of times to repeat the
                                      measure (default is 1, cannot
                                      be greater than 9)
                    Separated by ONE space

        Examples:   -       Begin new measure without repeat    |

                    - r     Begin OR End measure repeat         |: or :|

                    - R     End AND Begin measure repeats       :|:

                    - r 4   End measure, repeat 4 times         4
                                                               --+
                                                                :|
                                                               --+

    New Time Signature:
        t <upper_value> <lower_value>       --> Separated by ONE space
            upper_value = Upper Time Sig value between 0 and 99
            lower_value = Lower Time Sig value between 0 and 99

            This can only be entered after a New Measure line.

                    Example:   Time 5
                                    -  is enter as:  t 5 8
                                    8

    Tie:
        + [<dur>]   dur = Tied note duration ** OPTIONAL **
                    Separated by ONE space

        Examples:   +     Tie the NEXT note with the previous note

                    + e   Tie an eighth note of the same string and fret


    Chords:         c       The next note is part of the chord with the
                            previous note.

                                  e
                            Ex: +---+  is entered as:  3 5 e
                                |-5-|                  c
                                |-3-|                  2 3 e
                                +---+

    Triplet:        ^       Ex:    e^e^e   is entered as:  3 5 e
                                 +-------+                 ^
                                 |-5-4-3-|                 3 4 e
                                 |-------|                 ^
                                 +-------+                 3 3 e


    Bend:           B or b

                                   e
                            Ex:  +----+  is entered as:  3 5 e
                                 |-5^-|                  B
                                 |----|
                                 +----+


    Trill:          ~

                                   e
                            Ex:  +----+  is entered as:  3 5 e
                                 |-5~-|                  ~
                                 |----|
                                 +----+


    Slide to fret:  /       Slide up to fret
                    \       Slide down to fret

                                   e e e e
                            Ex:  +---------+  is entered as:  3 3 e
                                 |-3/5-----|                  /
                                 |-----7\5-|                  3 5 e
                                 +---------+                  2 7 e
                                                              \
                                                              2 5 e

    Slide to nowhere: /X    Slide up from fret: Not to another fret
                      \X    Slide down from fret: Not to another fret
                            ** Upper or lower case X accepted **

                      X     Same as /X    ** These two remain for   **
                      x     Same as \X    ** backward compatibility **

                                   e   e   e
                            Ex:  +-------------+  is entered as:  3 9 e
                                 |-9/X-----4\X-|                  /X
                                 |-----7\X-----|                  2 7 e
                                 +-------------+                  \x
                                                                  3 4 e
                                                                  x

    Slide from nowhere: X/  Slide up to fret (not from another fret)
                        X\  Slide down to fret (not from another fret)
                            ** Upper or lower case X accepted **

                                     e   e
                            Ex:  +---------+  is entered as:  X/
                                 |-X/9-----|                  3 9 e
                                 |-----X\7-|                  x\
                                 +---------+                  2 7 e

    Hammer On:      H or h  Hit fret of first note, then hammer on to next note

                                   e e
                            Ex:  +-----+  is entered as:  3 5 e
                                 |-5H7-|                  h
                                 |-----|                  3 7 e
                                 +-----+

    Pull Off:       P or p  Hit fret of first note, then pull off to next note

                                   e e
                            Ex:  +-----+  is entered as:  3 4 e
                                 |-4P2-|                  p
                                 |-----|                  3 2 e
                                 +-----+

******************************************************************************/

#include <stdio.h>

/*******************************
*    DO NOT CHANGE THESE
*******************************/
#ifndef TRUE
#   define TRUE         1
#   define FALSE        0
#endif

#ifndef ON
#   define ON           1
#   define OFF          0
#endif

#define NO_CUSTOM_FONT  0
#define CUSTOM_FONT_1   1
#define CUSTOM_FONT_2   2

#define UNIX            0
#define MSDOS           1
#define ATARI_ST        2

#define DEBUG           FALSE
/******************************/


/*
 *  Set the value of MACHINE to the appropriate value.
 *  This is used mainly for file name conventions.
 */
#define MACHINE UNIX


/*
 *  The duration characters consist of two parts (top and bottom).  This was
 *  done so that a custom character set can be used to print them using two
 *  characters, one on top of the other (for better looking output).
 *
 *  When FONT = NO_CUSTOM_FONT, the durations are printing using the
 *  letters w, W, h, H, etc. The top half in this case is just a space
 *  character.
 *
 *  When FONT = CUSTOM_FONT_1 or CUSTOM_FONT_2, alternate characters
 *  can be printed. This is so one can create a custom character set with the
 *  characters below defined to look like musical notes.
 */
#define FONT NO_CUSTOM_FONT


/*
 *  If using CUSTOM FONT 1 or 2 and an Epson (or compatible) printer,
 *  the line spacing between the two parts of the duration will be
 *  changed to make the note characters touch in the output.
 *
 *  TRUE = Using an Epson       FALSE = NOT using an Epson
 */
#define EPSON TRUE


/* Maximum line length allowed is 200 characters. */
/* Minimum line length allowed is 40 characters.  */
#define MAX_LINE_LENGTH     200
#define MIN_LINE_LENGTH     40

/*
 *  Default output line length. This is the page width of the output.
 *  The line length is also adjustable through a command line option.
 *
 *  minimum=MIN_LINE_LENGTH, maximum=MAX_LINE_LENGTH (defined below)
 */
#define DEFAULT_LINE_LENGTH  80


#if MACHINE == UNIX
#   define MAX_FILENAME_LEN    30    /* My limits, not Unix's */
#   define MAX_BASENAME_LEN    26
#else /* MACHINE == MSDOS or ATARI_ST */
#   define MAX_FILENAME_LEN    12    /* Add 1 for the period */
#   define MAX_BASENAME_LEN    8
#endif

/* Used internally by the program */
#define TYPE_FIRST          0
#define TYPE_NOTE           1
#define TYPE_MEASURE        2
#define TYPE_CHORD          3
#define TYPE_REST           4
#define TYPE_TIE            5
#define TYPE_SLIDE_DOWN     6
#define TYPE_SLIDE_UP       7

#define MAX_INPUT_LINE      80
#define MAX_OUTPUT_LINE     80


#define OPEN_MODE           "w"     /* Mode to open output (.tab) file */

/*
    These defines are for the ordering of the string arrays.
*/
#define INFO_T              0   /* Top info line for anything
                                   (eg. top of double height notes)     */
#define INFO_B              1   /* Bottom info line for durations, etc. */
#define S4                  2   /* String 4 is the HIGHER pitched       */
#define S3                  3
#define S2                  4
#define S1                  5   /* String 1 is the LOWER pitched        */


#define NUM_STRINGS         6   /* 4 Bass strings + 2 INFO lines        */


/*
    The following lines define the characters that will be detected for the
    different features.  As an alternative for STRING_1_DETECT to
    STRING_4_DETECT, use the strings' note values: E-A-D-G instead of 1-2-3-4.

    The strings are positioned in the output like this:

        4 (G string) +-----+
        3 (D string) |-----|
        2 (A string) |-----|
        1 (E string) +-----+
*/
#define STRING_1_DETECT                 '1'     /* 'E' */
#define STRING_2_DETECT                 '2'     /* 'A' */
#define STRING_3_DETECT                 '3'     /* 'D' */
#define STRING_4_DETECT                 '4'     /* 'G' */

#define COMMENT_DETECT                  '#'
#define MEASURE_DETECT                  '-'
#define REPEAT_DETECT                   'r'
#define REPEAT_AGAIN_DETECT             'R'
#define TIE_DETECT                      '+'
#define TRIPLET_DETECT                  '^'
#define CHORD_1_DETECT                  'C'
#define CHORD_2_DETECT                  'c'
#define SLIDE_UP_FRET_DETECT            '/'
#define SLIDE_DOWN_FRET_DETECT          '\\'
#define SLIDE_NOWHERE_1_DETECT          'X'
#define SLIDE_NOWHERE_2_DETECT          'x'
#define HAMMERON_1_DETECT               'H'
#define HAMMERON_2_DETECT               'h'
#define PULLOFF_1_DETECT                'P'
#define PULLOFF_2_DETECT                'p'
#define TRILL_DETECT                    '~'
#define BEND_1_DETECT                   'B'
#define BEND_2_DETECT                   'b'
#define TIMESIG_1_DETECT                'T'
#define TIMESIG_2_DETECT                't'
#define REST_1_DETECT                   'R'
#define REST_2_DETECT                   'r'
#define PAREN_NOTE_1_DETECT             '('
#define PAREN_NOTE_2_DETECT             ')'
#define WHOLE_DETECT                    'w'
#define WHOLE_DOT_DETECT                'W'
#define HALF_DETECT                     'h'
#define HALF_DOT_DETECT                 'H'
#define QUARTER_DETECT                  'q'
#define QUARTER_DOT_DETECT              'Q'
#define EIGHTH_DETECT                   'e'
#define EIGHTH_DOT_DETECT               'E'
#define SIXTEENTH_DETECT                's'
#define SIXTEENTH_DOT_DETECT            'S'
#define THIRTY_SECOND_DETECT            't'
#define THIRTY_SECOND_DOT_DETECT        'T'
#define SIXTY_FOURTH_DETECT             'x'
#define SIXTY_FOURTH_DOT_DETECT         'X'


/*
    The following lines define the characters that will be printed for the
    different durations, notes, etc.  This is here so that if one is using a
    custom char set and would like other characters to be printed, they can
    be redefined here. Three character sets are available here. By changing
    FONT, one of the characters sets will be printed.

    The first group of #define lines is for print characters common to all
    character sets.
*/
#define STAFF_START_INFO_T              " "
#define STAFF_START_INFO_B              " "
#define STAFF_START_S4                  "+"
#define STAFF_START_S3                  "|"
#define STAFF_START_S2                  "|"
#define STAFF_START_S1                  "+"

#define SPACE_PRINT_CHAR                ' '
#define UNDERLINE_PRINT_CHAR            '='   /* For underlining the title */

#define MEASURE_S4_PRINT_CHAR           '+'
#define MEASURE_S3_PRINT_CHAR           '|'
#define MEASURE_S2_PRINT_CHAR           '|'
#define MEASURE_S1_PRINT_CHAR           '+'

#define TAB_SPACE_S4_PRINT_CHAR         '-'
#define TAB_SPACE_S3_PRINT_CHAR         '-'
#define TAB_SPACE_S2_PRINT_CHAR         '-'
#define TAB_SPACE_S1_PRINT_CHAR         '-'

#define REPEAT_S4_PRINT_CHAR            '-'
#define REPEAT_S3_PRINT_CHAR            '*'
#define REPEAT_S2_PRINT_CHAR            '*'
#define REPEAT_S1_PRINT_CHAR            '-'

#define TRIPLET_PRINT_CHAR              '^'
#define SLIDE_UP_PRINT_CHAR             '/'
#define SLIDE_DOWN_PRINT_CHAR           '\\'
#define NOWHERE_PRINT_CHAR              'X'
#define HAMMERON_ON_PRINT_CHAR          'H'
#define PULLOFF_PRINT_CHAR              'P'
#define TRILL_PRINT_CHAR                '~'
#define BEND_PRINT_CHAR                 '^'
#define TOP_SIGNATURE_PRINT_CHAR        ':'
#define BOT_SIGNATURE_PRINT_CHAR        ':'
#define REST_PRINT_CHAR                 'R'
#define START_PAREN_PRINT_CHAR          '('
#define END_PAREN_PRINT_CHAR            ')'


/*
 *  Define characters for the different character sets.
 */

#if FONT == NO_CUSTOM_FONT

#   define TOP_WHOLE_PRINT_CHAR                 SPACE_PRINT_CHAR
#   define BOT_WHOLE_PRINT_CHAR                 'w'
#   define TOP_WHOLE_DOT_PRINT_CHAR             SPACE_PRINT_CHAR
#   define BOT_WHOLE_DOT_PRINT_CHAR             'W'
#   define TOP_HALF_PRINT_CHAR                  SPACE_PRINT_CHAR
#   define BOT_HALF_PRINT_CHAR                  'h'
#   define TOP_HALF_DOT_PRINT_CHAR              SPACE_PRINT_CHAR
#   define BOT_HALF_DOT_PRINT_CHAR              'H'
#   define TOP_QUARTER_PRINT_CHAR               SPACE_PRINT_CHAR
#   define BOT_QUARTER_PRINT_CHAR               'q'
#   define TOP_QUARTER_DOT_PRINT_CHAR           SPACE_PRINT_CHAR
#   define BOT_QUARTER_DOT_PRINT_CHAR           'Q'
#   define TOP_EIGHTH_PRINT_CHAR                SPACE_PRINT_CHAR
#   define BOT_EIGHTH_PRINT_CHAR                'e'
#   define TOP_EIGHTH_DOT_PRINT_CHAR            SPACE_PRINT_CHAR
#   define BOT_EIGHTH_DOT_PRINT_CHAR            'E'
#   define TOP_SIXTEENTH_PRINT_CHAR             SPACE_PRINT_CHAR
#   define BOT_SIXTEENTH_PRINT_CHAR             's'
#   define TOP_SIXTEENTH_DOT_PRINT_CHAR         SPACE_PRINT_CHAR
#   define BOT_SIXTEENTH_DOT_PRINT_CHAR         'S'
#   define TOP_THIRTY_SECOND_PRINT_CHAR         SPACE_PRINT_CHAR
#   define BOT_THIRTY_SECOND_PRINT_CHAR         't'
#   define TOP_THIRTY_SECOND_DOT_PRINT_CHAR     SPACE_PRINT_CHAR
#   define BOT_THIRTY_SECOND_DOT_PRINT_CHAR     'T'
#   define TOP_SIXTY_FOURTH_PRINT_CHAR          SPACE_PRINT_CHAR
#   define BOT_SIXTY_FOURTH_PRINT_CHAR          'x'
#   define TOP_SIXTY_FOURTH_DOT_PRINT_CHAR      SPACE_PRINT_CHAR
#   define BOT_SIXTY_FOURTH_DOT_PRINT_CHAR      'X'

#   define TIE_PRINT_CHAR                       '+'

/*
#else
#  if FONT == CUSTOM_FONT_1
*/
#elif FONT == CUSTOM_FONT_1

#   define TOP_WHOLE_PRINT_CHAR                 SPACE_PRINT_CHAR
#   define BOT_WHOLE_PRINT_CHAR                 '!'
#   define TOP_WHOLE_DOT_PRINT_CHAR             SPACE_PRINT_CHAR
#   define BOT_WHOLE_DOT_PRINT_CHAR             '@'
#   define TOP_HALF_PRINT_CHAR                  SPACE_PRINT_CHAR
#   define BOT_HALF_PRINT_CHAR                  '`'
#   define TOP_HALF_DOT_PRINT_CHAR              SPACE_PRINT_CHAR
#   define BOT_HALF_DOT_PRINT_CHAR              ','
#   define TOP_QUARTER_PRINT_CHAR               SPACE_PRINT_CHAR
#   define BOT_QUARTER_PRINT_CHAR               '#'
#   define TOP_QUARTER_DOT_PRINT_CHAR           SPACE_PRINT_CHAR
#   define BOT_QUARTER_DOT_PRINT_CHAR           '.'
#   define TOP_EIGHTH_PRINT_CHAR                SPACE_PRINT_CHAR
#   define BOT_EIGHTH_PRINT_CHAR                '$'
#   define TOP_EIGHTH_DOT_PRINT_CHAR            SPACE_PRINT_CHAR
#   define BOT_EIGHTH_DOT_PRINT_CHAR            '{'
#   define TOP_SIXTEENTH_PRINT_CHAR             SPACE_PRINT_CHAR
#   define BOT_SIXTEENTH_PRINT_CHAR             '%'
#   define TOP_SIXTEENTH_DOT_PRINT_CHAR         SPACE_PRINT_CHAR
#   define BOT_SIXTEENTH_DOT_PRINT_CHAR         ';'
#   define TOP_THIRTY_SECOND_PRINT_CHAR         SPACE_PRINT_CHAR
#   define BOT_THIRTY_SECOND_PRINT_CHAR         '&'
#   define TOP_THIRTY_SECOND_DOT_PRINT_CHAR     SPACE_PRINT_CHAR
#   define BOT_THIRTY_SECOND_DOT_PRINT_CHAR     '}'
#   define TOP_SIXTY_FOURTH_PRINT_CHAR          SPACE_PRINT_CHAR
#   define BOT_SIXTY_FOURTH_PRINT_CHAR          '['
#   define TOP_SIXTY_FOURTH_DOT_PRINT_CHAR      SPACE_PRINT_CHAR
#   define BOT_SIXTY_FOURTH_DOT_PRINT_CHAR      ']'

#   define TIE_PRINT_CHAR                       '_'

/*
#  else
#  if FONT == CUSTOM_FONT_2
*/
#elif FONT == CUSTOM_FONT_2

#   define TOP_WHOLE_PRINT_CHAR                 SPACE_PRINT_CHAR
#   define BOT_WHOLE_PRINT_CHAR                 '!'
#   define TOP_WHOLE_DOT_PRINT_CHAR             SPACE_PRINT_CHAR
#   define BOT_WHOLE_DOT_PRINT_CHAR             ';'
#   define TOP_HALF_PRINT_CHAR                  '#'
#   define BOT_HALF_PRINT_CHAR                  '$'
#   define TOP_HALF_DOT_PRINT_CHAR              '#'
#   define BOT_HALF_DOT_PRINT_CHAR              '{'
#   define TOP_QUARTER_PRINT_CHAR               '#'
#   define BOT_QUARTER_PRINT_CHAR               '%'
#   define TOP_QUARTER_DOT_PRINT_CHAR           '#'
#   define BOT_QUARTER_DOT_PRINT_CHAR           '}'
#   define TOP_EIGHTH_PRINT_CHAR                '&'
#   define BOT_EIGHTH_PRINT_CHAR                '%'
#   define TOP_EIGHTH_DOT_PRINT_CHAR            '&'
#   define BOT_EIGHTH_DOT_PRINT_CHAR            '}'
#   define TOP_SIXTEENTH_PRINT_CHAR             '`'
#   define BOT_SIXTEENTH_PRINT_CHAR             '%'
#   define TOP_SIXTEENTH_DOT_PRINT_CHAR         '`'
#   define BOT_SIXTEENTH_DOT_PRINT_CHAR         '}'
#   define TOP_THIRTY_SECOND_PRINT_CHAR         '`'
#   define BOT_THIRTY_SECOND_PRINT_CHAR         '@'
#   define TOP_THIRTY_SECOND_DOT_PRINT_CHAR     '`'
#   define BOT_THIRTY_SECOND_DOT_PRINT_CHAR     '['
#   define TOP_SIXTY_FOURTH_PRINT_CHAR          ','
#   define BOT_SIXTY_FOURTH_PRINT_CHAR          '.'
#   define TOP_SIXTY_FOURTH_DOT_PRINT_CHAR      ','
#   define BOT_SIXTY_FOURTH_DOT_PRINT_CHAR      ']'

#   define TIE_PRINT_CHAR                       '_'

/*#  endif /* FONT == CUSTOM_FONT_1 */
#endif /* FONT == NO_CUSTOM_FONT */

/* End of print characters*/


/*
 *  Typedefs and other defines
 */
typedef unsigned int  UINT;
typedef unsigned char UBYTE;
typedef char          CHAR;
typedef int           INT;
typedef void          VOID;

#define NULL_CHAR     '\0'


/*
 *  Macro to print the measure
 */
#define PRINT_MEASURE       fprintf( outfile, "%s\n%s\n%s\n%s\n%s\n%s\n\n", \
                                prt_strings[INFO_T], prt_strings[INFO_B],   \
                                prt_strings[S4], prt_strings[S3],           \
                                prt_strings[S2], prt_strings[S1] )

/*
 *  Macro to null out the input line
 */
#define CLEAR_INPUT_LINE    for ( i = 0; i < MAX_INPUT_LINE; i++ ) \
                                inputline[i] = '\0'

/*
 *  Macro to null out the print lines
 */
#define CLEAR_PRINT_LINE    prt_strings[INFO_T][0] = '\0'; \
                            prt_strings[INFO_B][0] = '\0'; \
                            prt_strings[S4][0]     = '\0'; \
                            prt_strings[S3][0]     = '\0'; \
                            prt_strings[S2][0]     = '\0'; \
                            prt_strings[S1][0]     = '\0'

/*
 *  Macro to null out the string info
 */
#define CLEAR_STRINGS       for ( i = 0; i <= MAX_OUTPUT_LINE; i++ ) \
                            {                                        \
                                strings[INFO_T][i] = '\0';           \
                                strings[INFO_B][i] = '\0';           \
                                strings[S4][i]     = '\0';           \
                                strings[S3][i]     = '\0';           \
                                strings[S2][i]     = '\0';           \
                                strings[S1][i]     = '\0';           \
                            }

/*
 *  Macro to toggle "x" between 1 and 0 ( or ON and OFF )
 */
#define TOGGLE(x)       (x) = 1 - (x)

/*
 *  Macro to add a vertical bar to the measure
 */
#define TAB_BAR         strings[INFO_T][pos] = SPACE_PRINT_CHAR;      \
                        strings[INFO_B][pos] = SPACE_PRINT_CHAR;      \
                        strings[S4][pos]     = MEASURE_S4_PRINT_CHAR; \
                        strings[S3][pos]     = MEASURE_S3_PRINT_CHAR; \
                        strings[S2][pos]     = MEASURE_S2_PRINT_CHAR; \
                        strings[S1][pos]     = MEASURE_S1_PRINT_CHAR

/*
 *  Macro to add a "space" to the measure
 */
#define TAB_SPACE       strings[INFO_T][pos] = SPACE_PRINT_CHAR;        \
                        strings[INFO_B][pos] = SPACE_PRINT_CHAR;        \
                        strings[S4][pos]     = TAB_SPACE_S4_PRINT_CHAR; \
                        strings[S3][pos]     = TAB_SPACE_S3_PRINT_CHAR; \
                        strings[S2][pos]     = TAB_SPACE_S2_PRINT_CHAR; \
                        strings[S1][pos]     = TAB_SPACE_S1_PRINT_CHAR

/*
 *  Macro to add a "repeat" to the measure
 */
#define TAB_REPEAT      strings[INFO_T][pos] = SPACE_PRINT_CHAR;     \
                        strings[INFO_B][pos] = SPACE_PRINT_CHAR;     \
                        strings[S4][pos]     = REPEAT_S4_PRINT_CHAR; \
                        strings[S3][pos]     = REPEAT_S3_PRINT_CHAR; \
                        strings[S2][pos]     = REPEAT_S2_PRINT_CHAR; \
                        strings[S1][pos]     = REPEAT_S1_PRINT_CHAR

/*
 *  Macro to end a measure with NULLs
 */
#define TAB_END         strings[INFO_T][pos] = '\0'; \
                        strings[INFO_B][pos] = '\0'; \
                        strings[S4][pos]     = '\0'; \
                        strings[S3][pos]     = '\0'; \
                        strings[S2][pos]     = '\0'; \
                        strings[S1][pos]     = '\0'

/*
 *  This macro converts the character representing the
 *  string that was read from the file to a number
 *  to access the "strings" array (S1 to S4).
 */
#define STRING_CHAR_TO_NUMBER(c,n)  switch( c )             \
                                    {                       \
                                      case STRING_1_DETECT: \
                                        n = S1; break;      \
                                      case STRING_2_DETECT: \
                                        n = S2; break;      \
                                      case STRING_3_DETECT: \
                                        n = S3; break;      \
                                      case STRING_4_DETECT: \
                                        n = S4; break;      \
                                      default:              \
                                        n = S1; break;      \
                                    }

/*
 *  Macros to print error messages and exit
 */
#define PRINT_ERROR(s,v)     fprintf( stderr, s, v )
#define PRINT_ERROR_F(s,f,v) fprintf( stderr, s, f, v )
#define ERROR_EXIT(s)        { fprintf( stderr, s ); exit(1); }
#define ERROR_EXIT_V(s,v)    { fprintf( stderr, s, v ); exit(1); }
#define ERROR_EXIT_FV(s,f,v) { fprintf( stderr, s, f, v ); exit(1); }

/*
 *  Macro to print usage message
 */
#define PRINT_USAGE     fprintf( stderr, \
                "\n  USAGE: %s [-?] [-l line_length] <input_files[.btc]>\n\n",\
                        argv[0] )

/*
 *  GLOBALS
 */
UINT max_line_length;       /* Length of line in characters   */
     num_of_lines_read;     /* Number of lines read from file */

/*
 *  If using CUSTOM FONT 1 or 2 and an Epson printer,
 *  define control codes to change the line spacing.
 */
#if ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 ) && EPSON == TRUE
      /* Control codes to change Epson printer line spacing */
      UBYTE EIGHT_PT_SPACING[] = { 27, 65, 8, 0 }; /* 8/72" Spacing */
      UBYTE NORMAL_SPACING[]   = { 27, 50, 0 };    /* 1/6"  Spacing */
#endif


main( argc, argv )
INT     argc;
CHAR    *argv[];
{
    CHAR    *parmameter;
    INT     parm_index,
            option;

    max_line_length = DEFAULT_LINE_LENGTH;

    /*
        Process all specified files
    */
    for ( parm_index = 1; parm_index < argc; parm_index++ )
    {
        if ( argv[parm_index][0] == '-' )
        {
            if ( argv[parm_index][1] == 'l' )      /* LINE LENGTH CHANGE */
            {
                if ( strlen( argv[parm_index] ) > 2 )
                    parmameter = (char *) &argv[parm_index][2];
                else
                {
                    if ( argc < 3 )
                    {
                        ERROR_EXIT(
                          "\n  ERROR: A line length must be supplied.\n\n" );
                    }

                    parmameter = (char *) argv[++parm_index];
                }

                max_line_length = atoi(parmameter);

                if ( max_line_length > MAX_LINE_LENGTH )
                    ERROR_EXIT_V( "\n  ERROR: Maximum line length is %d.\n\n",
                                  MAX_LINE_LENGTH );
                if ( max_line_length < MIN_LINE_LENGTH )
                    ERROR_EXIT_V( "\n  ERROR: Minimum line length is %d.\n\n",
                                  MIN_LINE_LENGTH );

                printf( "  New line length: %d\n", max_line_length );
            }
            else if ( argv[parm_index][1] == '?' )
                PRINT_USAGE;
            else
                ERROR_EXIT_V( "\n  ERROR: Invalid option %s.\n\n",
                              argv[parm_index] );
        } /* end if ( argv[parm_index][0] == '-' ) */
        else
            process_btc_file( argv[parm_index] );

    } /* end for */

} /* end of main() */


/*
 *  This function processes the BTC file.
 */
process_btc_file( btc_file_name )
CHAR *btc_file_name;            /* Name of current file being processed  */
{
    FILE    *infile,            /* Input file descriptor                 */
            *outfile;           /* Output file descriptor                */

    CHAR    /* Name of input and output file name */
            infile_name[MAX_FILENAME_LEN+1],
            outfile_name[MAX_FILENAME_LEN+1],

            /* Line from info file */
            inputline[MAX_INPUT_LINE+1],

            /* Strings (Bass Strings + Info Line) */
            strings[NUM_STRINGS][MAX_OUTPUT_LINE+1],

            /* Strings to Print */
            prt_strings[NUM_STRINGS][MAX_LINE_LENGTH+1],

            string_char,        /* Char representing string from file    */
            dur,                /* Note Duration (W=Whole, H=Half, etc.) */
            repeat_type,        /* Type of repeat (r or R)               */
            dummy;              /* Dummy char. Used in sscanf's          */

    INT     fret,               /* Fret number                           */
            i,                  /* Used in for loops                     */
            just_printed,       /* Used to tell if a measure was printed */
            last_type,          /* Was last item a note or measure       */
            num_params,         /* Number of parameters read by scanf    */
            pos,                /* Position in string                    */
            prt_pos,            /* Position in print string              */
            repeat,             /* Measure Repeat                        */
            string,             /* String number                         */
            ts_top,             /* Time Signature - Top value            */
            ts_bot,             /* Time Signature - Bottom value         */
            num_repeats,        /* Number of times a measure is repeated */
            fret_char_size;     /* Size of fret in chars (used for chords) */

    VOID    duration_to_print_chars(),
            print_tab_info();


    repeat = OFF;
    just_printed = FALSE;
    num_of_lines_read = 0;
    fret_char_size = 1;

    /*
     *  Create input and output file names.
     *  The input file must have a ".btc" extension which will
     *  be changed to ".tab" to create the output filename.
     *  Specifying the ".btc" extension on the command line
     *  is optional.
     */

    if ( strlen( btc_file_name ) > MAX_FILENAME_LEN )
        ERROR_EXIT_V( "ERROR: %s is an invalid filename. Name too long.\n\n",
                      btc_file_name )

    strcpy( infile_name, btc_file_name );
    strcpy( outfile_name, btc_file_name );

    for(    i = 0;
            ( infile_name[i] != '\0' ) && ( i < MAX_BASENAME_LEN );
            i++ )
    {
        if ( infile_name[i] == '.' )
            break;

    } /* end for */

    infile_name[i]   = '.';
    infile_name[i+1] = 'b';
    infile_name[i+2] = 't';
    infile_name[i+3] = 'c';
    infile_name[i+4] = '\0';
    outfile_name[i]   = '.';
    outfile_name[i+1] = 't';
    outfile_name[i+2] = 'a';
    outfile_name[i+3] = 'b';
    outfile_name[i+4] = '\0';

    /*
     *  Open input file
     */
    if ( ( infile = fopen( infile_name, "r" )) == (FILE *) NULL )
        ERROR_EXIT_V( "ERROR: Input file %s could not be opened\n\n",
                      infile_name );

    /*
     *  Open output file
     */
    if ( ( outfile = fopen( outfile_name, OPEN_MODE )) == (FILE *) NULL )
    {
        fclose( infile );
        ERROR_EXIT_V( "ERROR: Output file %s could not be opened.\n\n",
                      outfile_name );
    }

    printf( "  Processing file %s\n", infile_name );

    /*
     *  Read in song title and artist.
     *  Use prt_strings[INFO_B] as a buffer.
     */
    if ( fgets( prt_strings[INFO_B], max_line_length, infile ) != NULL )
    {
        num_of_lines_read++;

        fprintf( outfile, "\n%s", prt_strings[INFO_B] );

        /* Underline header */
        for ( i = 0; prt_strings[INFO_B][i] != '\n'; i++ )
            prt_strings[INFO_B][i] = UNDERLINE_PRINT_CHAR;

        fprintf( outfile, "%s\n", prt_strings[INFO_B] );
    }

    CLEAR_INPUT_LINE;

    CLEAR_PRINT_LINE;

    CLEAR_STRINGS;

    /*
     *  Start TAB with a vertical bar and a tab space.
     */
    prt_pos = 0;
    pos = 0;
    TAB_BAR;
    pos++;
    TAB_SPACE;

    last_type = TYPE_FIRST;

    /*
     *  LOOP:
     *    Read in a line of data from the input file
     *    Scan the line for "detection" characters
     */
    while ( fgets( inputline, MAX_INPUT_LINE, infile ) != NULL )
    {
        num_of_lines_read++;

        pos++;
        just_printed = FALSE;

        switch ( inputline[0] )
        {
            case COMMENT_DETECT:
                pos--;    /* To negate increment in while loop */
                break;

            case MEASURE_DETECT:
                num_params = sscanf( inputline, "%c %c %d",
                                     &dummy, &repeat_type, &num_repeats );

                if ( last_type != TYPE_FIRST )
                {
                    /*
                     *  Check for End Measure Repeat
                     */
                    if ( ( inputline[2] == REPEAT_DETECT && repeat == ON ) ||
                         ( inputline[2] == REPEAT_AGAIN_DETECT ) )
                    {
                        if ( num_params == 3 )
                        {
                            if ( ( num_repeats < 0 ) || ( num_repeats > 99 ) )
                            {
                                PRINT_ERROR_F(
                                    "ERROR in file %s: Number of repeats too large at line %d.\n",
                                    infile_name, num_of_lines_read );
                            }

                            if ( num_repeats > 9 )
                            {
                                TAB_SPACE;
                                strings[INFO_B][pos++] =
                                    ( num_repeats / 10 ) + '0';
                            }

                            TAB_REPEAT;
                            strings[INFO_B][pos++] = ( num_repeats % 10 ) + '0';

                        }
                        else
                        {
                            TAB_REPEAT;
                            pos++;
                        }

                    } /*  end if ( ( inputline[2] == REPEAT_DETECT ...  */

                    TAB_BAR;
                    pos++;

                    TAB_END;

                    if ( ( prt_pos + pos ) > max_line_length )
                    {

/* If CUSTOM FONT 1 or 2, change lines spacing for printing notes */
#if ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 ) && EPSON == TRUE
                        strcat( prt_strings[INFO_T], EIGHT_PT_SPACING );
                        strcat( prt_strings[INFO_B], NORMAL_SPACING );
#endif

                        PRINT_MEASURE;
                        CLEAR_PRINT_LINE;
                        just_printed = TRUE;

                        strcpy( prt_strings[INFO_T], STAFF_START_INFO_T );
                        strcpy( prt_strings[INFO_B], STAFF_START_INFO_B );
                        strcpy( prt_strings[S4],     STAFF_START_S4 );
                        strcpy( prt_strings[S3],     STAFF_START_S3 );
                        strcpy( prt_strings[S2],     STAFF_START_S2 );
                        strcpy( prt_strings[S1],     STAFF_START_S1 );

                        prt_pos = 1;
                    }

                    strcat( prt_strings[INFO_T], strings[INFO_T] );
                    strcat( prt_strings[INFO_B], strings[INFO_B] );
                    strcat( prt_strings[S4],     strings[S4] );
                    strcat( prt_strings[S3],     strings[S3] );
                    strcat( prt_strings[S2],     strings[S2] );
                    strcat( prt_strings[S1],     strings[S1] );

                    prt_pos += strlen( strings[INFO_B] );

                    pos = 0;
#if DEBUG == TRUE
                    CLEAR_STRINGS;
#endif

                }
                else /* if last_type == FIRST */
                    pos--;

                /*
                 *  Check for Begin Measure Repeat
                 */
                if (( inputline[2] == REPEAT_DETECT && repeat == OFF ) ||
                    ( inputline[2] == REPEAT_AGAIN_DETECT ))
                {
                    TAB_REPEAT;
                    pos++;
                }

                TAB_SPACE;

                if ( inputline[2] == REPEAT_DETECT )
                    TOGGLE(repeat);

                last_type = TYPE_MEASURE;

                break;

            case TIE_DETECT:
                if ( ( last_type == TYPE_NOTE ) ||
                     ( last_type == TYPE_REST ) ||
                     ( last_type == TYPE_TIE  ) )
                    pos--;

                num_params = sscanf( inputline, "%c %c", &dummy, &dur );

                TAB_SPACE;
                strings[INFO_B][pos] = TIE_PRINT_CHAR;

                if ( num_params == 2 )
                {
                    pos++;
                    TAB_SPACE;

                    duration_to_print_chars( dur, &strings[INFO_T][pos],
                                                  &strings[INFO_B][pos] );
                    pos++;

                    TAB_SPACE;
                }

                last_type = TYPE_TIE;

                break;

            case TRIPLET_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

/*
                TAB_SPACE;
*/

                strings[INFO_B][pos] = TRIPLET_PRINT_CHAR;

                break;

            case CHORD_1_DETECT:
            case CHORD_2_DETECT:
                if ( last_type == TYPE_NOTE )
                {
                    pos -= ( 2 + fret_char_size );
                    last_type = TYPE_CHORD;
                }
                break;

            case SLIDE_UP_FRET_DETECT:
            case SLIDE_DOWN_FRET_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

                TAB_SPACE;
                strings[string][pos] = inputline[0];

                /*
                 *  Check for Slide To Nowhere
                 */
                if ( inputline[1] == SLIDE_NOWHERE_1_DETECT ||
                     inputline[1] == SLIDE_NOWHERE_2_DETECT )
                {
                    pos++;
                    TAB_SPACE;
                    strings[string][pos++] = NOWHERE_PRINT_CHAR;
                    TAB_SPACE;
                }

                break;


            case SLIDE_NOWHERE_1_DETECT:
            case SLIDE_NOWHERE_2_DETECT:
                if ( inputline[1] == SLIDE_DOWN_FRET_DETECT )
                {
                    last_type = TYPE_SLIDE_DOWN;  /* Save info for later */
                }
                else if ( inputline[1] == SLIDE_UP_FRET_DETECT )
                {
                    last_type = TYPE_SLIDE_UP;    /* Save info for later */
                }
                else
                {
                    if ( last_type == TYPE_NOTE )
                        pos--;

                    TAB_SPACE;
                    strings[string][pos++] =
                        (inputline[0] == SLIDE_NOWHERE_1_DETECT) ?
                            SLIDE_UP_PRINT_CHAR : SLIDE_DOWN_PRINT_CHAR;
                    TAB_SPACE;
                    strings[string][pos++] = NOWHERE_PRINT_CHAR;
                }

                TAB_SPACE;

                break;


            case HAMMERON_1_DETECT:
            case HAMMERON_2_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

/*
                TAB_SPACE;
*/

                strings[string][pos] = HAMMERON_ON_PRINT_CHAR;

                break;


            case PULLOFF_1_DETECT:
            case PULLOFF_2_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

/*
                TAB_SPACE;
*/

                strings[string][pos] = PULLOFF_PRINT_CHAR;

                break;

            case TRILL_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

                TAB_SPACE;
                strings[string][pos++] = TRILL_PRINT_CHAR;
                TAB_SPACE;

                break;

            case BEND_1_DETECT:
            case BEND_2_DETECT:
                if ( last_type == TYPE_NOTE )
                    pos--;

                TAB_SPACE;
                strings[string][pos++] = BEND_PRINT_CHAR;
                TAB_SPACE;

                break;

            case TIMESIG_1_DETECT:
            case TIMESIG_2_DETECT:
                if ((last_type == TYPE_MEASURE) || (last_type == TYPE_FIRST))
                {
                    pos--;

                    sscanf( inputline, "%c %d %d", &dummy, &ts_top, &ts_bot );

                    if ( ( ts_top < 0 ) || ( ts_top > 99 ) ||
                         ( ts_bot < 0 ) || ( ts_bot > 99 ) )
                    {
                        PRINT_ERROR_F(
                            "ERROR in file %s: Invalid time signature at line %d.\n",
                            infile_name, num_of_lines_read );
                        break;
                    }

                    if ( ( ts_top > 10 ) || ( ts_bot > 10 ) )
                    {
                        TAB_SPACE;
                        strings[S3][pos]   = ( ts_top > 10 ) ?
                                               ( ts_top / 10 ) + '0' :
                                               SPACE_PRINT_CHAR;
                        strings[S2][pos++] = ( ts_bot > 10 ) ?
                                               ( ts_bot / 10 ) + '0' :
                                               SPACE_PRINT_CHAR;
                    }

                    TAB_SPACE;
                    strings[S3][pos]   = ( ts_top % 10 ) + '0';
                    strings[S2][pos++] = ( ts_bot % 10 ) + '0';

                    TAB_SPACE;
                    strings[S2][pos]   =  TOP_SIGNATURE_PRINT_CHAR;
                    strings[S3][pos++] =  BOT_SIGNATURE_PRINT_CHAR;

                    TAB_SPACE;
                }
                else
                    PRINT_ERROR_F(
                        "ERROR in file %s: Time signature at line %d not after a measure.\n",
                        infile_name, num_of_lines_read );
                break;

            case REST_1_DETECT:
            case REST_2_DETECT:
                sscanf( inputline, "%c %c", &dummy, &dur );

                TAB_SPACE;
                strings[S3][pos] = REST_PRINT_CHAR;

                duration_to_print_chars( dur, &strings[INFO_T][pos],
                                              &strings[INFO_B][pos] );
                pos++;

                TAB_SPACE;

                last_type = TYPE_REST;

                break;

            case PAREN_NOTE_1_DETECT:
            case PAREN_NOTE_2_DETECT:
                string = fret = 0;
                dur = '\0';

                num_params = sscanf( inputline,
                                     "%c %c %d %c",
                                     &dummy, &string_char, &fret, &dur );

                STRING_CHAR_TO_NUMBER( string_char, string );

                if ( num_params == 2 )
                    dur = NULL_CHAR;
                else if ( num_params < 2 )
                {
                    fclose( infile );
                    fclose( outfile );
                    ERROR_EXIT_FV(
                        "ERROR in file %s: Invalid note at line %d. PROGRAM EXITING!\n\n",
                        infile_name,
                        num_of_lines_read );
                }

                TAB_SPACE;
                strings[string][pos++] = START_PAREN_PRINT_CHAR;
                TAB_SPACE;

                if ( ( fret / 10 ) > 0 )      /* Check if fret number > 9 */
                    strings[string][pos++] = ( fret / 10 ) + '0';

                duration_to_print_chars( dur, &strings[INFO_T][pos],
                                              &strings[INFO_B][pos] );

                strings[string][pos++] = ( fret % 10 ) + '0';

                TAB_SPACE;
                strings[string][pos++] = END_PAREN_PRINT_CHAR;
                TAB_SPACE;

                last_type = TYPE_NOTE;

                break;

            case STRING_1_DETECT:                       /* NOTES */
            case STRING_2_DETECT:
            case STRING_3_DETECT:
            case STRING_4_DETECT:
                string = fret = 0;
                dur = '\0';
                fret_char_size = 1;

                num_params = sscanf( inputline,
                                     "%c %d %c", &string_char, &fret, &dur );

                STRING_CHAR_TO_NUMBER( string_char, string );

                if ( num_params == 2 )
                    dur = NULL_CHAR;
                else if ( num_params < 2 )
                {
                    fclose( infile );
                    fclose( outfile );
                    ERROR_EXIT_FV(
                        "ERROR in file %s: Invalid note at line %d. PROGRAM EXITING!\n\n",
                        infile_name,
                        num_of_lines_read );
                }

                if ( last_type == TYPE_SLIDE_DOWN )
                {
                    TAB_SPACE;
                    strings[string][pos++] = NOWHERE_PRINT_CHAR;
                    TAB_SPACE;
                    strings[string][pos++] = SLIDE_DOWN_PRINT_CHAR;
                }
                else if ( last_type == TYPE_SLIDE_UP )
                {
                    TAB_SPACE;
                    strings[string][pos++] = NOWHERE_PRINT_CHAR;
                    TAB_SPACE;
                    strings[string][pos++] = SLIDE_UP_PRINT_CHAR;
                }

                if ( last_type != TYPE_CHORD )
                {
                    TAB_SPACE;
                }

                if ( ( fret / 10 ) > 0 )      /* Check if fret number > 9 */
                {
                    fret_char_size = 2;

                    strings[INFO_B][pos]   = ' ';
                    strings[string][pos++] = ( fret / 10 ) + '0';

                    if ( last_type != TYPE_CHORD )
                    {
                       TAB_SPACE;
                    }
                }

                duration_to_print_chars( dur, &strings[INFO_T][pos],
                                              &strings[INFO_B][pos] );

                strings[string][pos++] = ( fret % 10 ) + '0';

                TAB_SPACE;

                last_type = TYPE_NOTE;

                break;

            default:
                PRINT_ERROR_F(
                    "ERROR in file %s: Unrecognized character at line %d\n",
                    infile_name, num_of_lines_read );

        } /* end switch */

        CLEAR_INPUT_LINE;

    } /* end while */

    if ( just_printed == FALSE )
    {
/* If CUSTOM FONT 1 or 2, change lines spacing for printing notes */
#if ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 ) && EPSON == TRUE
                        strcat( prt_strings[INFO_T], EIGHT_PT_SPACING );
                        strcat( prt_strings[INFO_B], NORMAL_SPACING );
#endif

        PRINT_MEASURE;
    }


#if DEBUG == FALSE
    /*
     *  Print tab notes.
     */
    print_tab_info( outfile );
#endif

    /*
     *  Close the files.
     */
    fclose( infile );
    fclose( outfile );

} /* end of process_btc_file */


/*
 *  Function duration_to_print_chars
 *
 *  This function fills in the duration values
 *  according to the duration value.
 *
 */
VOID duration_to_print_chars( dur, top_half, bottom_half )
CHAR    dur,
        *top_half,
        *bottom_half;
{
    switch( dur )
    {
        case WHOLE_DETECT:
            *top_half    = TOP_WHOLE_PRINT_CHAR;
            *bottom_half = BOT_WHOLE_PRINT_CHAR;
            break;

        case WHOLE_DOT_DETECT:
            *top_half    = TOP_WHOLE_DOT_PRINT_CHAR;
            *bottom_half = BOT_WHOLE_DOT_PRINT_CHAR;
            break;

        case HALF_DETECT:
            *top_half    = TOP_HALF_PRINT_CHAR;
            *bottom_half = BOT_HALF_PRINT_CHAR;
            break;

        case HALF_DOT_DETECT:
            *top_half    = TOP_HALF_DOT_PRINT_CHAR;
            *bottom_half = BOT_HALF_DOT_PRINT_CHAR;
            break;

        case QUARTER_DETECT:
            *top_half    = TOP_QUARTER_PRINT_CHAR;
            *bottom_half = BOT_QUARTER_PRINT_CHAR;
            break;

        case QUARTER_DOT_DETECT:
            *top_half    = TOP_QUARTER_DOT_PRINT_CHAR;
            *bottom_half = BOT_QUARTER_DOT_PRINT_CHAR;
            break;

        case EIGHTH_DETECT:
            *top_half    = TOP_EIGHTH_PRINT_CHAR;
            *bottom_half = BOT_EIGHTH_PRINT_CHAR;
            break;

        case EIGHTH_DOT_DETECT:
            *top_half    = TOP_EIGHTH_DOT_PRINT_CHAR;
            *bottom_half = BOT_EIGHTH_DOT_PRINT_CHAR;
            break;

        case SIXTEENTH_DETECT:
            *top_half    = TOP_SIXTEENTH_PRINT_CHAR;
            *bottom_half = BOT_SIXTEENTH_PRINT_CHAR;
            break;

        case SIXTEENTH_DOT_DETECT:
            *top_half    = TOP_SIXTEENTH_DOT_PRINT_CHAR;
            *bottom_half = BOT_SIXTEENTH_DOT_PRINT_CHAR;
            break;

        case THIRTY_SECOND_DETECT:
            *top_half    = TOP_THIRTY_SECOND_PRINT_CHAR;
            *bottom_half = BOT_THIRTY_SECOND_PRINT_CHAR;
            break;

        case THIRTY_SECOND_DOT_DETECT:
            *top_half    = TOP_THIRTY_SECOND_DOT_PRINT_CHAR;
            *bottom_half = BOT_THIRTY_SECOND_DOT_PRINT_CHAR;
            break;

        case SIXTY_FOURTH_DETECT:
            *top_half    = TOP_SIXTY_FOURTH_PRINT_CHAR;
            *bottom_half = BOT_SIXTY_FOURTH_PRINT_CHAR;
            break;

        case SIXTY_FOURTH_DOT_DETECT:
            *top_half    = TOP_SIXTY_FOURTH_DOT_PRINT_CHAR;
            *bottom_half = BOT_SIXTY_FOURTH_DOT_PRINT_CHAR;
            break;

        case NULL_CHAR:
            *top_half    = SPACE_PRINT_CHAR;
            *bottom_half = SPACE_PRINT_CHAR;
            break;

        default:
            *top_half    = SPACE_PRINT_CHAR;
            *bottom_half = SPACE_PRINT_CHAR;

            PRINT_ERROR(
                "ERROR in file: Unrecognized duration at line %d\n",
                num_of_lines_read );
            break;

        } /* end switch */

} /* end duration_to_print_chars */


/*
 *  Print tab notes.
 *
 *  This is info written at the end of the output file
 *  to describe the meaning of the symbols used.
 *
 *  If using FONT 1 or 2 and EPSON is set to TRUE, it prints
 *  a different trailer due to changing the line spacing.
 */
VOID print_tab_info( outfile )
FILE *outfile;
{

fprintf( outfile, "\
==========================================================================\n\n" );

#if ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 ) && EPSON == TRUE

fprintf( outfile, "\
 Created using \"The Bass Tab Creator\"  by:  Mario D'Alessio\n\n" );

fprintf( outfile, "\
 Notes:  %c          %c         %c%s\n\
         %c = Whole  %c = Half  %c = Quarter%s\n\n",
TOP_WHOLE_PRINT_CHAR, TOP_HALF_PRINT_CHAR, TOP_QUARTER_PRINT_CHAR,
EIGHT_PT_SPACING,
BOT_WHOLE_PRINT_CHAR, BOT_HALF_PRINT_CHAR, BOT_QUARTER_PRINT_CHAR,
NORMAL_SPACING );

fprintf( outfile, "\
         %c          %c         %c         %c%s\n\
         %c = 8th    %c = 16th  %c = 32nd  %c = 64th%s\n\n",
TOP_EIGHTH_PRINT_CHAR, TOP_SIXTEENTH_PRINT_CHAR,
TOP_THIRTY_SECOND_PRINT_CHAR, TOP_SIXTY_FOURTH_PRINT_CHAR,
EIGHT_PT_SPACING,
BOT_EIGHTH_PRINT_CHAR, BOT_SIXTEENTH_PRINT_CHAR,
BOT_THIRTY_SECOND_PRINT_CHAR, BOT_SIXTY_FOURTH_PRINT_CHAR,
NORMAL_SPACING );

fprintf( outfile, "\
      n\n\
  +----+                     +----+\n\
  |%c--%c+  = MEASURE          |4%c--|  = TIME\n\
  |%c--%c+    REPEATS          |4%c--|    SIGNATURES\n\
  +----+   (n=Num Repeats)   +----+\n\n",
REPEAT_S3_PRINT_CHAR, REPEAT_S3_PRINT_CHAR, TOP_SIGNATURE_PRINT_CHAR,
REPEAT_S2_PRINT_CHAR, REPEAT_S2_PRINT_CHAR, BOT_SIGNATURE_PRINT_CHAR );
    
fprintf( outfile, "\
  %c = TIE         %c = BEND              %c  = TRIPLET\n\
  %c = HAMMER ON   %c = SLIDE TO FRET     %c%c = SLIDE TO NOWHERE\n\
  %c = PULL OFF    %c = REST              %c  = TRILL\n",
TIE_PRINT_CHAR, BEND_PRINT_CHAR, TRIPLET_PRINT_CHAR,
HAMMERON_ON_PRINT_CHAR, SLIDE_DOWN_PRINT_CHAR, SLIDE_UP_PRINT_CHAR,
NOWHERE_PRINT_CHAR, PULLOFF_PRINT_CHAR, REST_PRINT_CHAR, TRILL_PRINT_CHAR );


#else    /* Not using an EPSON with FONT 1 or 2 */


fprintf( outfile, "\
          Created using \"The Bass Tab Creator\" by Mario D'Alessio\n\
     __\n\
   +|oo|+       Notes:  %c          %c         %c\n\
   +|oo|+               %c = Whole  %c = Half  %c = Quarter\n",
TOP_WHOLE_PRINT_CHAR, TOP_HALF_PRINT_CHAR, TOP_QUARTER_PRINT_CHAR,
BOT_WHOLE_PRINT_CHAR, BOT_HALF_PRINT_CHAR, BOT_QUARTER_PRINT_CHAR );

#if ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 )
    fprintf( outfile, "\
     ||\n\
     ||                 %c          %c         %c         %c\n",
    TOP_EIGHTH_PRINT_CHAR, TOP_SIXTEENTH_PRINT_CHAR,
    TOP_THIRTY_SECOND_PRINT_CHAR, TOP_SIXTY_FOURTH_PRINT_CHAR );
#endif

fprintf( outfile, "\
     ||                 %c = 8th    %c = 16th  %c = 32nd  %c = 64th\n\
     ||\n",
BOT_EIGHTH_PRINT_CHAR, BOT_SIXTEENTH_PRINT_CHAR,
BOT_THIRTY_SECOND_PRINT_CHAR, BOT_SIXTY_FOURTH_PRINT_CHAR );

#if FONT == NO_CUSTOM_FONT
    fprintf( outfile, "\
     ||                 Capitol notes are dotted notes (1.5 x duration)\n\
     ||\n" );
#endif

fprintf( outfile, "\
     ||             n\n\
     ||         +----+                     +----+\n\
     ||         |%c--%c+  = MEASURE          |4%c--|  = TIME\n\
  _  ||  _      |%c--%c+    REPEATS          |4%c--|    SIGNATURES\n\
  \\\\_||_//      +----+   (n=Num Repeats)   +----+\n",
REPEAT_S3_PRINT_CHAR, REPEAT_S3_PRINT_CHAR, TOP_SIGNATURE_PRINT_CHAR,
REPEAT_S2_PRINT_CHAR, REPEAT_S2_PRINT_CHAR, BOT_SIGNATURE_PRINT_CHAR );

fprintf( outfile, "\
   | [] |\n\
   | || |       %c = TIE         %c  = TRIPLET           %c = BEND\n",
TIE_PRINT_CHAR, TRIPLET_PRINT_CHAR, BEND_PRINT_CHAR );

fprintf( outfile, "\
  /  []  \\      %c = HAMMER ON   %c  = SLIDE TO FRET     %c = REST\n\
  \\______/      %c = PULL OFF    %c%c = SLIDE TO NOWHERE  %c = TRILL\n",
HAMMERON_ON_PRINT_CHAR, SLIDE_DOWN_PRINT_CHAR, REST_PRINT_CHAR,
PULLOFF_PRINT_CHAR, SLIDE_UP_PRINT_CHAR, NOWHERE_PRINT_CHAR, TRILL_PRINT_CHAR );

#endif /* ( FONT == CUSTOM_FONT_1 || FONT == CUSTOM_FONT_2 ) && EPSON == TRUE */

} /* end print_tab_info */
